package ram.tutorial.editor;

import java.util.ArrayList;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.text.reconciler.DirtyRegion;
import org.eclipse.jface.text.reconciler.IReconcilingStrategy;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;

import ram.tutorial.editor.outline.VGLOutlinePage;
import ram.tutorial.editor.parser.ASTFunctionDef;
import ram.tutorial.editor.parser.ASTStart;
import ram.tutorial.editor.parser.SimpleNode;
import ram.tutorial.editor.parser.Token;

/**
 * 
 * @author Ram Kulkarni (rakulkar@adobe.com)
 *
 */
public class VGLReconcilingStrategy implements IReconcilingStrategy {

	IDocument doc = null;
	VGLEditor editor = null;
	ISourceViewer sourceViewer = null;
	
	public void reconcile(IRegion partition) {
		updateEditor(-1,-1);
	}
	
	private void updateEditor(int offset, int length)
	{
		//we support only one partition
		
		if (editor == null)
			return;
		
		//parse the document 
		editor.parseDocument();
		
		VGLOutlinePage outlinePage = (VGLOutlinePage) editor.getAdapter(IContentOutlinePage.class);
		
		outlinePage.refreshTree();
		
		createFoldingPositions(offset, length);
		
		editor.redrawViewer();
	}

	private void createFoldingPositions(int offset, int length) {
		
		ASTStart startNode = editor.startNode;
		
		if (startNode == null)
			return;
		
		final ArrayList<Position> positions = new ArrayList<Position>();
		
		//We support code folding for function declarations only, which
		//appear at the top level
		
		for (int i = 0; i < startNode.jjtGetNumChildren(); i++)
		{
			SimpleNode childNode = (SimpleNode) startNode.jjtGetChild(i);
			
			if (childNode instanceof ASTFunctionDef)
			{
				Token endToken = ((ASTFunctionDef)childNode).getClosingCurlyBraceToken();
				if (endToken == null)
					continue;
				
				Token startToken = childNode.jjtGetFirstToken();
				
				try
				{
					int startOffset = doc.getLineOffset(startToken.beginLine-1) + startToken.beginColumn - 1;
					int endOffset = doc.getLineOffset(endToken.endLine-1) + endToken.endColumn ;
					
					if (offset >= 0 && length > 0)
					{
						if (!(startOffset >= offset && endOffset <= (offset + length)))
							continue;
					}
					
					Position pos = new Position(startOffset, (endOffset-startOffset));
					positions.add(pos);
				} catch (BadLocationException e) {}
			}
		}
		
		if (positions.size() > 0)
		{
			Display.getDefault().asyncExec(new Runnable() {

				public void run() {
					editor.updateFoldingStructure(positions);
					editor.redrawViewer();
				}
				
			});
		}
		
	}
	
	public void reconcile(DirtyRegion dirtyRegion, IRegion subRegion) {
		updateEditor(dirtyRegion.getOffset(), dirtyRegion.getLength());
	}

	public void setDocument(IDocument document) {
		doc = document;
		updateEditor(-1,-1);
	}

	public void setEditor(VGLEditor editor)
	{
		this.editor = editor;
	}
	
	public void setSourceViewer (ISourceViewer viewer)
	{
		this.sourceViewer = viewer;
	}
}
